@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008, 2017, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,8 +22,8 @@
  *
  */
 
-#ifndef CPU_ARM_VM_JNITYPES_ARM_HPP
-#define CPU_ARM_VM_JNITYPES_ARM_HPP
+#ifndef CPU_ARM_JNITYPES_ARM_HPP
+#define CPU_ARM_JNITYPES_ARM_HPP
 
 #include "jni.h"
 #include "memory/allocation.hpp"
@@ -79,4 +79,4 @@ public:
 
 };
 
-#endif // CPU_ARM_VM_JNITYPES_ARM_HPP
+#endif // CPU_ARM_JNITYPES_ARM_HPP
