@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,8 +22,8 @@
  *
  */
 
-#ifndef CPU_SPARC_VM_MACROASSEMBLER_SPARC_HPP
-#define CPU_SPARC_VM_MACROASSEMBLER_SPARC_HPP
+#ifndef CPU_SPARC_MACROASSEMBLER_SPARC_HPP
+#define CPU_SPARC_MACROASSEMBLER_SPARC_HPP
 
 #include "asm/assembler.hpp"
 #include "utilities/macros.hpp"
@@ -1397,4 +1397,4 @@ class SkipIfEqual : public StackObj {
    ~SkipIfEqual();
 };
 
-#endif // CPU_SPARC_VM_MACROASSEMBLER_SPARC_HPP
+#endif // CPU_SPARC_MACROASSEMBLER_SPARC_HPP
