@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 1997, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,8 +22,8 @@
  *
  */
 
-#ifndef CPU_SPARC_VM_MACROASSEMBLER_SPARC_INLINE_HPP
-#define CPU_SPARC_VM_MACROASSEMBLER_SPARC_INLINE_HPP
+#ifndef CPU_SPARC_MACROASSEMBLER_SPARC_INLINE_HPP
+#define CPU_SPARC_MACROASSEMBLER_SPARC_INLINE_HPP
 
 #include "asm/assembler.inline.hpp"
 #include "asm/macroAssembler.hpp"
@@ -719,4 +719,4 @@ inline void MacroAssembler::swap(const Address& a, Register d, int offset) {
   if (a.has_index()) { assert(offset == 0, ""); swap(a.base(), a.index(), d        ); }
   else               {                          swap(a.base(), a.disp() + offset, d); }
 }
-#endif // CPU_SPARC_VM_MACROASSEMBLER_SPARC_INLINE_HPP
+#endif // CPU_SPARC_MACROASSEMBLER_SPARC_INLINE_HPP
