@@ -124,9 +124,14 @@ class InterpreterMacroAssembler: public MacroAssembler {
   void load_resolved_reference_at_index(Register result, Register index, Register tmp = rscratch2);
 
   // load cpool->resolved_klass_at(index)
-  void load_resolved_klass_at_index(Register cpool,  // the constant pool (corrupted on return)
-                                    Register index,  // the constant pool index (corrupted on return)
-                                    Register klass); // contains the Klass on return
+  void load_resolved_klass_at_index(Register klass,  // contains the Klass on return
+                                    Register cpool,  // the constant pool (corrupted on return)
+                                    Register index); // the constant pool index (corrupted on return)
+
+  void load_resolved_method_at_index(int byte_no,
+                                     Register method,
+                                     Register cache,
+                                     Register index);
 
   NOT_LP64(void f2ieee();)        // truncate ftos to 32bits
   NOT_LP64(void d2ieee();)        // truncate dtos to 64bits
