@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2012, 2018 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -74,6 +74,18 @@ static char* create_standard_memory(size_t size) {
   return mapAddress;
 }
 
+// delete the PerfData memory region
+//
+static void delete_standard_memory(char* addr, size_t size) {
+
+  // there are no persistent external resources to cleanup for standard
+  // memory. since DestroyJavaVM does not support unloading of the JVM,
+  // cleanup of the memory resource is not performed. The memory will be
+  // reclaimed by the OS upon termination of the process.
+  //
+  return;
+}
+
 // save the specified memory region to the given file
 //
 // Note: this function might be called from signal handler (by os::abort()),
@@ -526,7 +538,7 @@ static char* get_user_name(uid_t uid) {
 
   char* pwbuf = NEW_C_HEAP_ARRAY(char, bufsize, mtInternal);
 
-  struct passwd* p = NULL;
+  struct passwd* p;
   int result = getpwuid_r(uid, &pwent, pwbuf, (size_t)bufsize, &p);
 
   if (result != 0 || p == NULL || p->pw_name == NULL || *(p->pw_name) == '\0') {
@@ -673,7 +685,7 @@ static char* get_user_name_slow(int vmid, TRAPS) {
           if (statbuf.st_ctime > oldest_ctime) {
             char* user = strchr(dentry->d_name, '_') + 1;
 
-            if (oldest_user != NULL) FREE_C_HEAP_ARRAY(char, oldest_user);
+            FREE_C_HEAP_ARRAY(char, oldest_user);
             oldest_user = NEW_C_HEAP_ARRAY(char, strlen(user)+1, mtInternal);
 
             strcpy(oldest_user, user);
@@ -737,15 +749,15 @@ static void remove_file(const char* path) {
   }
 }
 
-// Cleanup stale shared memory files
+// Cleanup stale shared memory resources
 //
 // This method attempts to remove all stale shared memory files in
 // the named user temporary directory. It scans the named directory
-// for files matching the pattern ^$[0-9]*$.
-//
-// This directory should be used only by JVM processes owned by the
-// current user to store PerfMemory files. Any other files found
-static void cleanup_sharedmem_files(const char* dirname) {
+// for files matching the pattern ^$[0-9]*$. For each file found, the
+// process id is extracted from the file name and a test is run to
+// determine if the process is alive. If the process is not alive,
+// any stale file resources are removed.
+static void cleanup_sharedmem_resources(const char* dirname) {
 
   int saved_cwd_fd;
   // Open the directory.
@@ -756,54 +768,47 @@ static void cleanup_sharedmem_files(const char* dirname) {
   }
 
   // For each entry in the directory that matches the expected file
-  // name pattern, remove the file if it's determine to be stale
-  // Note, instrumented HotSpot processes for this user may start and/or
-  // terminate during this search and remove or create new files in this
-  // directory. The behavior of this loop under these conditions is dependent
-  // upon the implementation of opendir/readdir.
+  // name pattern, determine if the file resources are stale and if
+  // so, remove the file resources. Note, instrumented HotSpot processes
+  // for this user may start and/or terminate during this search and
+  // remove or create new files in this directory. The behavior of this
+  // loop under these conditions is dependent upon the implementation of
+  // opendir/readdir.
   struct dirent* entry;
   errno = 0;
   while ((entry = os::readdir(dirp)) != NULL) {
 
-    const char* filename = entry->d_name;
-    pid_t pid = filename_to_pid(filename);
+    pid_t pid = filename_to_pid(entry->d_name);
 
     if (pid == 0) {
 
-      if (strcmp(filename, ".") != 0 && strcmp(filename, "..") != 0) {
+      if (strcmp(entry->d_name, ".") != 0 && strcmp(entry->d_name, "..") != 0) {
 
         // Attempt to remove all unexpected files, except "." and "..".
-        unlink(filename);
+        unlink(entry->d_name);
       }
 
       errno = 0;
       continue;
     }
 
-    // The following code assumes that pid must be in the same
-    // namespace as the current process.
-    bool stale = false;
+    // We now have a file name that converts to a valid integer
+    // that could represent a process id . if this process id
+    // matches the current process id or the process is not running,
+    // then remove the stale file resources.
+    //
+    // Process liveness is detected by sending signal number 0 to
+    // the process id (see kill(2)). if kill determines that the
+    // process does not exist, then the file resources are removed.
+    // if kill determines that that we don't have permission to
+    // signal the process, then the file resources are assumed to
+    // be stale and are removed because the resources for such a
+    // process should be in a different user specific directory.
+    if ((pid == os::current_process_id()) ||
+        (kill(pid, 0) == OS_ERR && (errno == ESRCH || errno == EPERM))) {
 
-    if (pid == os::current_process_id()) {
-      // The file was created by a terminated process that happened
-      // to have the same pid as the current process.
-      stale = true;
-    } else if (kill(pid, 0) == OS_ERR) {
-      if (errno == ESRCH) {
-        // The target process does not exist.
-        stale = true;
-      } else if (errno == EPERM) {
-        // The file was created by a terminated process that happened
-        // to have the same pid as a process not owned by the current user.
-        stale = true;
-      }
+        unlink(entry->d_name);
     }
-
-    if (stale) {
-      log_info(perf, memops)("Remove stale file %s/%s", dirname, filename);
-      unlink(filename);
-    }
-
     errno = 0;
   }
 
@@ -846,13 +851,13 @@ static bool make_user_tmp_dir(const char* dirname) {
   return true;
 }
 
-// create the shared memory file
+// create the shared memory file resources
 //
 // This method creates the shared memory file with the given size
 // This method also creates the user specific temporary directory, if
 // it does not yet exist.
 //
-static int create_sharedmem_file(const char* dirname, const char* filename, size_t size) {
+static int create_sharedmem_resources(const char* dirname, const char* filename, size_t size) {
 
   // make the user temporary directory
   if (!make_user_tmp_dir(dirname)) {
@@ -1008,13 +1013,12 @@ static char* mmap_create_shared(size_t size) {
   }
 
   // cleanup any stale shared memory files
-  cleanup_sharedmem_files(dirname);
+  cleanup_sharedmem_resources(dirname);
 
   assert(((size > 0) && (size % os::vm_page_size() == 0)),
          "unexpected PerfMemory region size");
 
-  log_info(perf, memops)("Trying to open %s/%s", dirname, short_filename);
-  fd = create_sharedmem_file(dirname, short_filename, size);
+  fd = create_sharedmem_resources(dirname, short_filename, size);
 
   FREE_C_HEAP_ARRAY(char, user_name);
   FREE_C_HEAP_ARRAY(char, dirname);
@@ -1047,8 +1051,6 @@ static char* mmap_create_shared(size_t size) {
   // It does not go through os api, the operation has to record from here.
   MemTracker::record_virtual_memory_reserve((address)mapAddress, size, CURRENT_PC, mtInternal);
 
-  log_info(perf, memops)("Successfully opened");
-
   return mapAddress;
 }
 
@@ -1074,10 +1076,10 @@ static char* create_shared_memory(size_t size) {
 //
 static void delete_shared_memory(char* addr, size_t size) {
 
-  // Remove the shared memory file. Since DestroyJavaVM does
-  // not support unloading of the JVM, unmapping of the memory region is
+  // cleanup the persistent shared memory resources. since DestroyJavaVM does
+  // not support unloading of the JVM, unmapping of the memory resource is
   // not performed. The memory will be reclaimed by the OS upon termination of
-  // the process.
+  // the process. The backing store file is deleted from the file system.
 
   assert(!PerfDisableSharedMem, "shouldn't be here");
 
@@ -1288,7 +1290,10 @@ void PerfMemory::delete_memory_region() {
     save_memory_to_file(start(), capacity());
   }
 
-  if (!PerfDisableSharedMem) {
+  if (PerfDisableSharedMem) {
+    delete_standard_memory(start(), capacity());
+  }
+  else {
     delete_shared_memory(start(), capacity());
   }
 }
