@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2013, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2012, 2013 SAP SE. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -23,8 +23,8 @@
  *
  */
 
-#ifndef OS_CPU_LINUX_PPC_VM_OS_LINUX_PPC_HPP
-#define OS_CPU_LINUX_PPC_VM_OS_LINUX_PPC_HPP
+#ifndef OS_CPU_LINUX_PPC_OS_LINUX_PPC_HPP
+#define OS_CPU_LINUX_PPC_OS_LINUX_PPC_HPP
 
   static void setup_fpu() {}
 
@@ -32,4 +32,4 @@
   // Note: Currently only used in 64 bit Windows implementations
   static bool register_code_area(char *low, char *high) { return true; }
 
-#endif // OS_CPU_LINUX_PPC_VM_OS_LINUX_PPC_HPP
+#endif // OS_CPU_LINUX_PPC_OS_LINUX_PPC_HPP
