@@ -485,10 +485,9 @@ bool ShortLoopOptimizer::process(BlockBegin* loop_header) {
 
 
 GlobalValueNumbering::GlobalValueNumbering(IR* ir)
-  : _current_map(NULL)
+  : _compilation(ir->compilation())
+  , _current_map(NULL)
   , _value_maps(ir->linear_scan_order()->length(), ir->linear_scan_order()->length(), NULL)
-  , _compilation(ir->compilation())
-  , _has_substitutions(false)
 {
   TRACE_VALUE_NUMBERING(tty->print_cr("****** start of global value numbering"));
 
