@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,8 +22,8 @@
  *
  */
 
-#ifndef SHARE_VM_MEMORY_CLASSLISTPARSER_HPP
-#define SHARE_VM_MEMORY_CLASSLISTPARSER_HPP
+#ifndef SHARE_CLASSFILE_CLASSLISTPARSER_HPP
+#define SHARE_CLASSFILE_CLASSLISTPARSER_HPP
 
 #include "utilities/exceptions.hpp"
 #include "utilities/globalDefinitions.hpp"
@@ -127,4 +127,4 @@ public:
   InstanceKlass* lookup_super_for_current_class(Symbol* super_name);
   InstanceKlass* lookup_interface_for_current_class(Symbol* interface_name);
 };
-#endif
+#endif // SHARE_CLASSFILE_CLASSLISTPARSER_HPP
