@@ -102,8 +102,17 @@ protected:
   }
 
   static unsigned oop_hash(oop const& s1) {
-    unsigned hash = (unsigned)((uintptr_t)&s1);
-    return hash ^ (hash >> LogMinObjAlignment);
+    // Robert Jenkins 1996 & Thomas Wang 1997
+    // http://web.archive.org/web/20071223173210/http://www.concentric.net/~Ttwang/tech/inthash.htm
+    uintptr_t tmp = cast_from_oop<uintptr_t>(s1);
+    unsigned hash = (unsigned)tmp;
+    hash = ~hash + (hash << 15);
+    hash = hash ^ (hash >> 12);
+    hash = hash + (hash << 2);
+    hash = hash ^ (hash >> 4);
+    hash = hash * 2057;
+    hash = hash ^ (hash >> 16);
+    return hash;
   }
 
   typedef ResourceHashtable<oop, ClassLoaderStats*,
