@@ -24,6 +24,7 @@
 
 #include "precompiled.hpp"
 #include "jvm.h"
+#include "classfile/symbolTable.hpp"
 #include "compiler/compilerOracle.hpp"
 #include "compiler/methodMatcher.hpp"
 #include "memory/allocation.inline.hpp"
@@ -379,10 +380,12 @@ static OracleCommand parse_command_name(const char * line, int* bytes_read) {
 
   *bytes_read = 0;
   char command[33];
-  int result = sscanf(line, "%32[a-z]%n", command, bytes_read);
-  for (uint i = 0; i < ARRAY_SIZE(command_names); i++) {
-    if (strcmp(command, command_names[i]) == 0) {
-      return (OracleCommand)i;
+  int matches = sscanf(line, "%32[a-z]%n", command, bytes_read);
+  if (matches > 0) {
+    for (uint i = 0; i < ARRAY_SIZE(command_names); i++) {
+      if (strcmp(command, command_names[i]) == 0) {
+        return (OracleCommand)i;
+      }
     }
   }
   return UnknownCommand;
@@ -828,8 +831,8 @@ void CompilerOracle::parse_compile_only(char * line) {
       }
 
       EXCEPTION_MARK;
-      Symbol* c_name = SymbolTable::new_symbol(className, CHECK);
-      Symbol* m_name = SymbolTable::new_symbol(methodName, CHECK);
+      Symbol* c_name = SymbolTable::new_symbol(className);
+      Symbol* m_name = SymbolTable::new_symbol(methodName);
       Symbol* signature = NULL;
 
       BasicMatcher* bm = new BasicMatcher();
