@@ -83,10 +83,6 @@ class CompilerOracle : AllStatic {
   static void parse_from_line(char* line);
   static void parse_compile_only(char * line);
 
-  // For updating the oracle file
-  static void append_comment_to_file(const char* message);
-  static void append_exclude_to_file(const methodHandle& method);
-
   // Tells whether there are any methods to print for print_method_statistics()
   static bool should_print_methods();
 };
