@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2016, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,8 +22,8 @@
  *
  */
 
-#ifndef SHARE_VM_GC_G1_G1YOUNGREMSETSAMPLINGTHREAD_HPP
-#define SHARE_VM_GC_G1_G1YOUNGREMSETSAMPLINGTHREAD_HPP
+#ifndef SHARE_GC_G1_G1YOUNGREMSETSAMPLINGTHREAD_HPP
+#define SHARE_GC_G1_G1YOUNGREMSETSAMPLINGTHREAD_HPP
 
 #include "gc/shared/concurrentGCThread.hpp"
 
@@ -63,4 +63,4 @@ public:
   double vtime_accum() { return _vtime_accum; }
 };
 
-#endif // SHARE_VM_GC_G1_G1YOUNGREMSETSAMPLINGTHREAD_HPP
+#endif // SHARE_GC_G1_G1YOUNGREMSETSAMPLINGTHREAD_HPP
