@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2014, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,8 +22,8 @@
  *
  */
 
-#ifndef SHARE_VM_GC_G1_HEAPREGIONTYPE_HPP
-#define SHARE_VM_GC_G1_HEAPREGIONTYPE_HPP
+#ifndef SHARE_GC_G1_HEAPREGIONTYPE_HPP
+#define SHARE_GC_G1_HEAPREGIONTYPE_HPP
 
 #include "gc/g1/g1HeapRegionTraceType.hpp"
 
@@ -196,4 +196,4 @@ public:
   static const HeapRegionType Humongous;
 };
 
-#endif // SHARE_VM_GC_G1_HEAPREGIONTYPE_HPP
+#endif // SHARE_GC_G1_HEAPREGIONTYPE_HPP
