@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2002, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2002, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,8 +22,8 @@
  *
  */
 
-#ifndef SHARE_VM_GC_PARALLEL_PSPROMOTIONMANAGER_INLINE_HPP
-#define SHARE_VM_GC_PARALLEL_PSPROMOTIONMANAGER_INLINE_HPP
+#ifndef SHARE_GC_PARALLEL_PSPROMOTIONMANAGER_INLINE_HPP
+#define SHARE_GC_PARALLEL_PSPROMOTIONMANAGER_INLINE_HPP
 
 #include "gc/parallel/parallelScavengeHeap.hpp"
 #include "gc/parallel/parMarkBitMap.inline.hpp"
@@ -375,4 +375,4 @@ void PSPromotionManager::record_steal(StarTask& p) {
 }
 #endif // TASKQUEUE_STATS
 
-#endif // SHARE_VM_GC_PARALLEL_PSPROMOTIONMANAGER_INLINE_HPP
+#endif // SHARE_GC_PARALLEL_PSPROMOTIONMANAGER_INLINE_HPP
