@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2001, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -22,8 +22,8 @@
  *
  */
 
-#ifndef SHARE_VM_GC_SHARED_GENOOPCLOSURES_INLINE_HPP
-#define SHARE_VM_GC_SHARED_GENOOPCLOSURES_INLINE_HPP
+#ifndef SHARE_GC_SHARED_GENOOPCLOSURES_INLINE_HPP
+#define SHARE_GC_SHARED_GENOOPCLOSURES_INLINE_HPP
 
 #include "gc/shared/cardTableRS.hpp"
 #include "gc/shared/genCollectedHeap.hpp"
@@ -171,4 +171,4 @@ inline void ScanWeakRefClosure::do_oop(narrowOop* p) { ScanWeakRefClosure::do_oo
 
 #endif // INCLUDE_SERIALGC
 
-#endif // SHARE_VM_GC_SHARED_GENOOPCLOSURES_INLINE_HPP
+#endif // SHARE_GC_SHARED_GENOOPCLOSURES_INLINE_HPP
