@@ -33,7 +33,7 @@ public:
     _alloc_shared_gc,   // Allocate common, outside of GCLAB
     _alloc_tlab,        // Allocate TLAB
     _alloc_gclab,       // Allocate GCLAB
-    _ALLOC_LIMIT,
+    _ALLOC_LIMIT
   };
 
   static const char* alloc_type_to_string(Type type) {
