@@ -47,9 +47,8 @@ public:
 
   void snap_before();
   void snap_after();
-  void print();
 
-  bool is_good_progress(const char *label);
+  bool is_good_progress();
 };
 
 #endif // SHARE_GC_SHENANDOAH_SHENANDOAHMETRICS_HPP
