@@ -51,6 +51,7 @@ class outputStream;
   f(scan_jfr_weak_roots,                            "    S: JFR Weak Roots")            \
   f(scan_jni_weak_roots,                            "    S: JNI Weak Roots")            \
   f(scan_stringtable_roots,                         "    S: String Table Roots")        \
+  f(scan_resolved_method_table_roots,               "    S: Resolved Table Roots")      \
   f(scan_vm_weak_roots,                             "    S: VM Weak Roots")             \
   f(scan_synchronizer_roots,                        "    S: Synchronizer Roots")        \
   f(scan_management_roots,                          "    S: Management Roots")          \
@@ -76,6 +77,7 @@ class outputStream;
   f(update_jfr_weak_roots,                          "    U: JFR Weak Roots")            \
   f(update_jni_weak_roots,                          "    U: JNI Weak Roots")            \
   f(update_stringtable_roots,                       "    U: String Table Roots")        \
+  f(update_resolved_method_table_roots,             "    U: Resolved Table Roots")      \
   f(update_vm_weak_roots,                           "    U: VM Weak Roots")             \
   f(update_synchronizer_roots,                      "    U: Synchronizer Roots")        \
   f(update_management_roots,                        "    U: Management Roots")          \
@@ -109,6 +111,7 @@ class outputStream;
   f(evac_jfr_weak_roots,                            "    E: JFR Weak Roots")            \
   f(evac_jni_weak_roots,                            "    E: JNI Weak Roots")            \
   f(evac_stringtable_roots,                         "    E: String Table Roots")        \
+  f(evac_resolved_method_table_roots,               "    E: Resolved Table Roots")      \
   f(evac_vm_weak_roots,                             "    E: VM Weak Roots")             \
   f(evac_synchronizer_roots,                        "    E: Synchronizer Roots")        \
   f(evac_management_roots,                          "    E: Management Roots")          \
@@ -139,6 +142,7 @@ class outputStream;
   f(final_update_jfr_weak_roots,                     "    UR: JFR Weak Roots")          \
   f(final_update_jni_weak_roots,                     "    UR: JNI Weak Roots")          \
   f(final_update_stringtable_roots,                  "    UR: String Table Roots")      \
+  f(final_update_resolved_method_table_roots,        "    UR: Resolved Table Roots")    \
   f(final_update_vm_weak_roots,                      "    UR: VM Weak Roots")           \
   f(final_update_refs_synchronizer_roots,            "    UR: Synchronizer Roots")      \
   f(final_update_refs_management_roots,              "    UR: Management Roots")        \
@@ -164,6 +168,7 @@ class outputStream;
   f(degen_gc_update_jfr_weak_roots,                  "    DU: JFR Weak Roots")          \
   f(degen_gc_update_jni_weak_roots,                  "    DU: JNI Weak Roots")          \
   f(degen_gc_update_stringtable_roots,               "    DU: String Table Roots")      \
+  f(degen_gc_update_resolved_method_table_roots,     "    DU: Resolved Table Roots")    \
   f(degen_gc_update_vm_weak_roots,                   "    DU: VM Weak Roots")           \
   f(degen_gc_update_synchronizer_roots,              "    DU: Synchronizer Roots")      \
   f(degen_gc_update_management_roots,                "    DU: Management Roots")        \
@@ -190,6 +195,7 @@ class outputStream;
   f(init_traversal_gc_jfr_weak_roots,                "    TI: JFR Weak Roots")          \
   f(init_traversal_gc_jni_weak_roots,                "    TI: JNI Weak Roots")          \
   f(init_traversal_gc_stringtable_roots,             "    TI: String Table Roots")      \
+  f(init_traversal_gc_resolved_method_table_roots,   "    TI: Resolved Table Roots")    \
   f(init_traversal_gc_vm_weak_roots,                 "    TI: VM Weak Roots")           \
   f(init_traversal_gc_synchronizer_roots,            "    TI: Synchronizer Roots")      \
   f(init_traversal_gc_management_roots,              "    TI: Management Roots")        \
@@ -213,6 +219,7 @@ class outputStream;
   f(final_traversal_gc_jfr_weak_roots,               "    TF: JFR Weak Roots")          \
   f(final_traversal_gc_jni_weak_roots,               "    TF: JNI Weak Roots")          \
   f(final_traversal_gc_stringtable_roots,            "    TF: String Table Roots")      \
+  f(final_traversal_gc_resolved_method_table_roots,  "    TF: Resolved Table Roots")    \
   f(final_traversal_gc_vm_weak_roots,                "    TF: VM Weak Roots")           \
   f(final_traversal_gc_synchronizer_roots,           "    TF: Synchronizer Roots")      \
   f(final_traversal_gc_management_roots,             "    TF: Management Roots")        \
@@ -225,24 +232,25 @@ class outputStream;
   f(final_traversal_gc_termination,                  "    TF:   Termination")           \
                                                                                         \
   /* Per-thread timer block, should have "roots" counters in consistent order */        \
-  f(final_traversal_update_roots,                    "  Update Roots")                  \
-  f(final_traversal_update_thread_roots,             "    TU: Thread Roots")            \
-  f(final_traversal_update_code_roots,               "    TU: Code Cache Roots")        \
-  f(final_traversal_update_universe_roots,           "    TU: Universe Roots")          \
-  f(final_traversal_update_jni_roots,                "    TU: JNI Roots")               \
-  f(final_traversal_update_jvmti_weak_roots,         "    TU: JVMTI Weak Roots")        \
-  f(final_traversal_update_jfr_weak_roots,           "    TU: JFR Weak Roots")          \
-  f(final_traversal_update_jni_weak_roots,           "    TU: JNI Weak Roots")          \
-  f(final_traversal_update_stringtable_roots,        "    TU: String Table Roots")      \
-  f(final_traversal_update_vm_weak_roots,            "    TU: VM Weak Roots")           \
-  f(final_traversal_update_synchronizer_roots,       "    TU: Synchronizer Roots")      \
-  f(final_traversal_update_management_roots,         "    TU: Management Roots")        \
-  f(final_traversal_update_system_dict_roots,        "    TU: System Dict Roots")       \
-  f(final_traversal_update_cldg_roots,               "    TU: CLDG Roots")              \
-  f(final_traversal_update_jvmti_roots,              "    TU: JVMTI Roots")             \
-  f(final_traversal_update_string_dedup_table_roots, "    TU: Dedup Table Roots")       \
-  f(final_traversal_update_string_dedup_queue_roots, "    TU: Dedup Queue Roots")       \
-  f(final_traversal_update_finish_queues,            "    TU: Finish Queues")           \
+  f(final_traversal_update_roots,                       "  Update Roots")               \
+  f(final_traversal_update_thread_roots,                "    TU: Thread Roots")         \
+  f(final_traversal_update_code_roots,                  "    TU: Code Cache Roots")     \
+  f(final_traversal_update_universe_roots,              "    TU: Universe Roots")       \
+  f(final_traversal_update_jni_roots,                   "    TU: JNI Roots")            \
+  f(final_traversal_update_jvmti_weak_roots,            "    TU: JVMTI Weak Roots")     \
+  f(final_traversal_update_jfr_weak_roots,              "    TU: JFR Weak Roots")       \
+  f(final_traversal_update_jni_weak_roots,              "    TU: JNI Weak Roots")       \
+  f(final_traversal_update_stringtable_roots,           "    TU: String Table Roots")   \
+  f(final_traversal_update_resolved_method_table_roots, "    TU: Resolved Table Roots") \
+  f(final_traversal_update_vm_weak_roots,               "    TU: VM Weak Roots")        \
+  f(final_traversal_update_synchronizer_roots,          "    TU: Synchronizer Roots")   \
+  f(final_traversal_update_management_roots,            "    TU: Management Roots")     \
+  f(final_traversal_update_system_dict_roots,           "    TU: System Dict Roots")    \
+  f(final_traversal_update_cldg_roots,                  "    TU: CLDG Roots")           \
+  f(final_traversal_update_jvmti_roots,                 "    TU: JVMTI Roots")          \
+  f(final_traversal_update_string_dedup_table_roots,    "    TU: Dedup Table Roots")    \
+  f(final_traversal_update_string_dedup_queue_roots,    "    TU: Dedup Queue Roots")    \
+  f(final_traversal_update_finish_queues,               "    TU: Finish Queues")        \
                                                                                         \
   f(traversal_gc_cleanup,                            "  Cleanup")                       \
                                                                                         \
@@ -261,6 +269,7 @@ class outputStream;
   f(full_gc_jfr_weak_roots,                          "    F: JFR Weak Roots")           \
   f(full_gc_jni_weak_roots,                          "    F: JNI Weak Roots")           \
   f(full_gc_stringtable_roots,                       "    F: String Table Roots")       \
+  f(full_gc_resolved_method_table_roots,             "    F: Resolved Table Roots")     \
   f(full_gc_vm_weak_roots,                           "    F: VM Weak Roots")            \
   f(full_gc_synchronizer_roots,                      "    F: Synchronizer Roots")       \
   f(full_gc_management_roots,                        "    F: Management Roots")         \
@@ -311,23 +320,24 @@ class outputStream;
   // end
 
 #define SHENANDOAH_GC_PAR_PHASE_DO(f)                           \
-  f(ThreadRoots,             "Thread Roots (ms):")              \
-  f(CodeCacheRoots,          "CodeCache Roots (ms):")           \
-  f(UniverseRoots,           "Universe Roots (ms):")            \
-  f(JNIRoots,                "JNI Handles Roots (ms):")         \
-  f(JVMTIWeakRoots,          "JVMTI Weak Roots (ms):")          \
-  f(JFRWeakRoots,            "JFR Weak Roots (ms):")            \
-  f(JNIWeakRoots,            "JNI Weak Roots (ms):")            \
-  f(StringTableRoots,        "StringTable Roots(ms):")          \
-  f(VMWeakRoots,             "VM Weak Roots(ms)")               \
-  f(ObjectSynchronizerRoots, "ObjectSynchronizer Roots (ms):")  \
-  f(ManagementRoots,         "Management Roots (ms):")          \
-  f(SystemDictionaryRoots,   "SystemDictionary Roots (ms):")    \
-  f(CLDGRoots,               "CLDG Roots (ms):")                \
-  f(JVMTIRoots,              "JVMTI Roots (ms):")               \
-  f(StringDedupTableRoots,   "String Dedup Table Roots (ms):")  \
-  f(StringDedupQueueRoots,   "String Dedup Queue Roots (ms):")  \
-  f(FinishQueues,            "Finish Queues (ms):")             \
+  f(ThreadRoots,              "Thread Roots (ms):")              \
+  f(CodeCacheRoots,           "CodeCache Roots (ms):")           \
+  f(UniverseRoots,            "Universe Roots (ms):")            \
+  f(JNIRoots,                 "JNI Handles Roots (ms):")         \
+  f(JVMTIWeakRoots,           "JVMTI Weak Roots (ms):")          \
+  f(JFRWeakRoots,             "JFR Weak Roots (ms):")            \
+  f(JNIWeakRoots,             "JNI Weak Roots (ms):")            \
+  f(StringTableRoots,         "StringTable Roots(ms):")          \
+  f(ResolvedMethodTableRoots, "Resolved Table Roots(ms):")       \
+  f(VMWeakRoots,              "VM Weak Roots(ms)")               \
+  f(ObjectSynchronizerRoots,  "ObjectSynchronizer Roots (ms):")  \
+  f(ManagementRoots,          "Management Roots (ms):")          \
+  f(SystemDictionaryRoots,    "SystemDictionary Roots (ms):")    \
+  f(CLDGRoots,                "CLDG Roots (ms):")                \
+  f(JVMTIRoots,               "JVMTI Roots (ms):")               \
+  f(StringDedupTableRoots,    "String Dedup Table Roots (ms):")  \
+  f(StringDedupQueueRoots,    "String Dedup Queue Roots (ms):")  \
+  f(FinishQueues,             "Finish Queues (ms):")             \
   // end
 
 class ShenandoahPhaseTimings : public CHeapObj<mtGC> {
