@@ -32,12 +32,10 @@ class JfrCheckpointWriter;
 class JfrTypeManager : public AllStatic {
  public:
   static bool initialize();
-  static void clear();
+  static void destroy();
   static void write_types(JfrCheckpointWriter& writer);
   static void write_safepoint_types(JfrCheckpointWriter& writer);
-  static void write_type_set();
-  static void write_type_set_for_unloaded_classes();
-  static void create_thread_checkpoint(JavaThread* jt);
+  static void create_thread_blob(JavaThread* jt);
   static void write_thread_checkpoint(JavaThread* jt);
 };
 
