@@ -178,6 +178,7 @@ bool JfrStackTrace::record_thread(JavaThread& thread, frame& frame) {
   u4 count = 0;
   _reached_root = true;
 
+  _hash = 1;
   while (!st.at_end()) {
     if (count >= _max_frames) {
       _reached_root = false;
@@ -199,7 +200,9 @@ bool JfrStackTrace::record_thread(JavaThread& thread, frame& frame) {
     }
     const int lineno = method->line_number_from_bci(bci);
     // Can we determine if it's inlined?
-    _hash = (_hash << 2) + (unsigned int)(((size_t)mid >> 2) + (bci << 4) + type);
+    _hash = (_hash * 31) + mid;
+    _hash = (_hash * 31) + bci;
+    _hash = (_hash * 31) + type;
     _frames[count] = JfrStackFrame(mid, bci, type, method);
     st.samples_next();
     count++;
@@ -235,6 +238,7 @@ bool JfrStackTrace::record_safe(JavaThread* thread, int skip) {
     vfs.next();
   }
 
+  _hash = 1;
   while (!vfs.at_end()) {
     if (count >= _max_frames) {
       _reached_root = false;
@@ -250,8 +254,9 @@ bool JfrStackTrace::record_safe(JavaThread* thread, int skip) {
     else {
       bci = vfs.bci();
     }
-    // Can we determine if it's inlined?
-    _hash = (_hash << 2) + (unsigned int)(((size_t)mid >> 2) + (bci << 4) + type);
+    _hash = (_hash * 31) + mid;
+    _hash = (_hash * 31) + bci;
+    _hash = (_hash * 31) + type;
     _frames[count] = JfrStackFrame(mid, bci, type, method);
     vfs.next();
     count++;
